@@ -5,8 +5,10 @@ import appdirs
 import os
 import sys
 import contextlib
-import fcntl
 import errno
+import warnings
+
+from .log import clog, log_context
 
 try:
     import cPickle as pickle
@@ -14,35 +16,87 @@ except ImportError:
     import pickle
 
 try:
-    unicode
-except NameError:
-    unicode = str       # aka, we are in Python 3.x
+    # See https://docs.python.org/3.6/library/msvcrt.html
+    import msvcrt
+
+    def _at_begin(file):
+        fpos = file.tell()
+        try:
+            file.seek(0, 0)
+            yield f
+        finally:
+            file.seek(fpos, 0)
+
+    def _acquire_flock_os(file, read_lock):
+        # Note: this has the side effect of chainging the file position
+        # and restoring it back during the lock-operation
+
+        # https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/locking?view=vs-2019
+        # seems that msvcrt has no support for read or shared locks
+        mode = msvcrt.LK_LOCK
+        while True:
+            try:
+                with _at_begin(file):
+                    msvcrt.locking(file.fileno(), mode, 1)
+                break
+            except OSError as err:
+                if err.errno in (errno.EACCES, errno.EDEADLOCK):
+                    continue
+                raise
+
+    def _release_flock_os(file):
+        # Note: this has the side effect of chainging the file position
+        # and restoring it back during the lock-operation
+        with _at_begin(file):
+            msvcrt.locking(file.fileno(), msvcrt.LK_UNLCK, 1)
+except ImportError:
+    pass
+
+try:
+    # See https://docs.python.org/3/library/fcntl.html#fcntl.flock
+    import fcntl
+
+    def _acquire_flock_os(file, read_lock):
+        fcntl.lockf(
+            file.fileno(), fcntl.LOCK_SH if read_lock else fcntl.LOCK_EX
+        )
+
+    def _release_flock_os(file):
+        fcntl.lockf(file.fileno(), fcntl.LOCK_UN)
+except ImportError:
+    pass
+
 
 @contextlib.contextmanager
-def flock(file, shared=False):
-    fcntl.lockf(file.fileno(), fcntl.LOCK_SH if shared else fcntl.LOCK_EX)
+def flock(file, read_lock=False):
+    _acquire_flock_os(file, read_lock)
     try:
         yield
     finally:
-        fcntl.lockf(file.fileno(), fcntl.LOCK_UN)
+        _release_flock_os(file)
+
 
 def create_file_new_or_fail(name):
-    if unicode == str:
-        # For Python 3.x with can open a file using 'x' flag.
-        # 'x' means create a new file or fail
-        # honestly, I'm not sure if 'x' is race-condition free
-        # this is a best effort implementation
-        return open(name, 'xb')
-    else:
-        # For Python 2.x we rollback to a no-atomic operation
-        # try to open for reading, if fails means that doesn't exist, good,
-        # create it then; if exists, fail
-        try:
-            open(name, 'rb').close()
-        except:
-            return open(name, 'wb')
+    # The 'x' means create a new file or fail
+    # Honestly, I'm not sure if 'x' is race-condition free
+    # this is a best effort implementation
+    return open(name, 'xb')
 
-        raise OSError(errno.EEXIST, "FileExistsError")
+
+'''
+>>> from byexample.log import init_log_system
+>>> init_log_system()
+
+>>> from byexample.cache import RegexCache
+>>> import warnings
+>>> warnings.filterwarnings('ignore', module='byexample.cache')
+
+Currently this feature does not work on Python 3.8
+>>> import sys
+>>> if sys.version_info <= (3, 7):
+...     print("cache enabled")
+<cache-enabled>
+'''
 
 
 class RegexCache(object):
@@ -52,6 +106,23 @@ class RegexCache(object):
         if self.disabled:
             return
 
+        warnings.warn(
+            "Cache is enabled. This is an *experimental* feature",
+            RuntimeWarning
+        )
+
+        try:
+            _acquire_flock_os
+            _release_flock_os
+        except NameError:
+            # if we cannot prevent race condition due a lack of file locks,
+            # disable the cache
+            warnings.warn(
+                "Cache will be disabled because the current OS/file system does not support file locks",
+                RuntimeWarning
+            )
+            self.disabled = True
+
         if filename:
             self.filename = self._cache_filepath(filename)
             self._cache = self._load_cache_from_disk()
@@ -60,7 +131,7 @@ class RegexCache(object):
             self._cache = self._new_cache()
 
         self.clear_stats()
-        self._log("Cache '%s': %i entries" % (self.filename, self._nkeys))
+        clog().chat("Cache '%s': %i entries", self.filename, self._nkeys)
 
     @contextlib.contextmanager
     def synced(self, label=""):
@@ -99,6 +170,7 @@ class RegexCache(object):
         finally:
             self._unpatch()
 
+    @log_context('byexample.cache')
     def clear_stats(self):
         self._nkeys, self._hits = len(self._cache), 0
 
@@ -109,7 +181,6 @@ class RegexCache(object):
             The path will be formed based on the user's cache directory,
             platform and python version.
 
-            >>> from byexample.cache import RegexCache
             >>> RegexCache._cache_filepath('foo')
             '<user-cache-dir>/byexample/re-<platform>-<python-version>/foo'
 
@@ -123,20 +194,11 @@ class RegexCache(object):
 
         version = "re-%s-%08i" % (sys.platform, sys.hexversion)
         dir = appdirs.user_cache_dir(appname='byexample', version=version)
-        try:
-            os.makedirs(dir)
-        except OSError as e:
-            # XXX: for Python 3.2 and greater, use exist_ok=True (see makedirs)
-            if e.errno != errno.EEXIST:
-                raise
+        os.makedirs(dir, exist_ok=True)
 
         filename = os.path.basename(filename)
         return os.path.join(dir, filename)
 
-    def _log(self, msg):
-        if self.verbose:
-            print(msg)
-
     def _load_cache_from_disk(self):
         ''' Load the cache from disk, create an empty one if the
             cache doesn't exist.
@@ -144,13 +206,10 @@ class RegexCache(object):
             If the load/read fails, return a empty cache too.
             '''
         try:
-            with open(self.filename, 'rb') as f, flock(f, shared=True):
+            with open(self.filename, 'rb') as f, flock(f, read_lock=True):
                 return self._read_cache_or_empty(f)
-        except IOError as e:
-            if e.errno == errno.ENOENT:    # aka Python 3.3's FileNotFoundError
-                return self._create_empty_cache_in_disk()
-            else:
-                raise
+        except FileNotFoundError as e:
+            return self._create_empty_cache_in_disk()
 
     def _read_cache_or_empty(self, file):
         ''' Read from the given file and load the cache.
@@ -166,7 +225,7 @@ class RegexCache(object):
             return pickle.loads(file.read())
         except:
             # possible corrupt cache, ignore it
-            self._log("Warning. Cache file '%s' corrupted." % self.filename)
+            clog().warn("Cache file '%s' corrupted.", self.filename)
             return self._new_cache()
 
     def _new_cache(self):
@@ -176,28 +235,30 @@ class RegexCache(object):
         ''' Create an empty cache in disk if doesn't exist yet. '''
         cache = self._new_cache()
         try:
-            self._log("Cache file '%s' does not exist. Creating a new one..." % self.filename)
+            clog().info(
+                "Cache file '%s' does not exist. Creating a new one...",
+                self.filename
+            )
             with create_file_new_or_fail(self.filename) as f, flock(f):
                 # the open didn't fail, so it *must* be new:
                 # save an empty cache
                 pickle.dump(cache, f)
-        except OSError as e:
-            if e.errno == errno.EEXIST:     # aka Python 3.3's FileExistsError
-                pass
-            else:
-                raise
+        except FileExistsError as e:
+            pass
 
         return cache
 
-
+    @log_context('byexample.cache')
     def _sync(self, label=""):
         misses = len(self._cache) - self._nkeys
         nohits = self._nkeys - self._hits
 
-        self._log("[%s] Cache stats: %i entries %i hits %i misses %i nohits." \
-                    % (label, len(self._cache), self._hits, misses, nohits))
+        clog().chat(
+            "[%s] Cache stats: %i entries %i hits %i misses %i nohits.", label,
+            len(self._cache), self._hits, misses, nohits
+        )
         if misses and self.filename != None:
-            self._log("[%s] Cache require sync." % label)
+            clog().chat("[%s] Cache require sync.", label)
             with open(self.filename, 'rb+') as f, flock(f):
                 # get a fresh disk version in case that other
                 # byexample instance had touched the cache but
@@ -211,7 +272,7 @@ class RegexCache(object):
 
                 # write to disk the new updated cache, truncate
                 # and shrink it if the new is smaller than the original.
-                f.seek(0,0)
+                f.seek(0, 0)
                 # XXX pickletools.optimize ??
                 pickle.dump(cache, f)
                 f.truncate()
@@ -235,13 +296,13 @@ class RegexCache(object):
                 >>> get = RegexCache(None).get
 
                 >>> r1 = re.compile(r'foo.*bar', re.DOTALL)
-                >>> r2 = get(r'foo.*bar', re.DOTALL)
+                >>> r2 = get(r'foo.*bar', re.DOTALL)     # byexample: +if=cache-enabled
 
-                >>> r1.pattern == r2.pattern
+                >>> r1.pattern == r2.pattern       # byexample: +if=cache-enabled
                 True
 
-                >>> r3 = re.compile(r2) # from another regex
-                >>> r4 = get(r2)  # but we don't support this
+                >>> r3 = re.compile(r2) # from another regex         # byexample: +if=cache-enabled 
+                >>> r4 = get(r2)        # but we don't support this  # byexample: +if=cache-enabled 
                 Traceback <...>
                 <...>
                 ValueError: Regex pattern must be a string or bytes but it is <...>
@@ -249,9 +310,11 @@ class RegexCache(object):
             RegexCache.get uses internal, undocumented functions from re module.
 
         '''
-        if not isinstance(pattern, (str, bytes, unicode)):
-            raise ValueError("Regex pattern must be a string or bytes but it is %s"
-                                % type(pattern))
+        if not isinstance(pattern, (str, bytes)):
+            raise ValueError(
+                "Regex pattern must be a string or bytes but it is %s" %
+                type(pattern)
+            )
 
         key = (pattern, flags)
         try:
@@ -264,9 +327,11 @@ class RegexCache(object):
         return self._bytecode_to_regex(pattern, bytecode)
 
     def _pattern_to_bytecode(self, pattern, flags=0):
-        if not isinstance(pattern, (str, bytes, unicode)):
-            raise ValueError("Regex pattern must be a string or bytes but it is %s"
-                                % type(pattern))
+        if not isinstance(pattern, (str, bytes)):
+            raise ValueError(
+                "Regex pattern must be a string or bytes but it is %s" %
+                type(pattern)
+            )
 
         p = sre_parse.parse(pattern, flags)
         code = [i.real for i in sre_compile._code(p, flags)]
@@ -284,13 +349,12 @@ class RegexCache(object):
         for k, i in groupindex.items():
             indexgroup[i] = k
 
-        indexgroup = tuple(indexgroup) # required in Python 3.7 to be a tuple
+        indexgroup = tuple(indexgroup)  # required in Python 3.7 to be a tuple
         return sre_compile._sre.compile(
-            pattern, flags, code,
-            ngroups-1,
-            groupindex, indexgroup
+            pattern, flags, code, ngroups - 1, groupindex, indexgroup
         )
 
+    @log_context('byexample.cache')
     def _patch(self):
 
         # PATCH! TODO is a better way?!?
@@ -299,6 +363,6 @@ class RegexCache(object):
 
         return self
 
+    @log_context('byexample.cache')
     def _unpatch(self, *args, **kargs):
         sre_compile.compile = self._original__sre_compile__compile
-
