@@ -268,6 +268,13 @@ def get_default_options_parser(cmdline_args):
         "amount of characters that must precede at minimum/maximum an input tag in the form min:max"
     )
 
+    options_parser.add_flag(
+        "force-echo-filtering",
+        default=False,
+        help=
+        "each interpreter disables the echo from the terminal but in some cases this cannot be done and an active filtering is required (this is an experimental feature, it will break your tests if no echo is received and it will force a full terminal emulation (see +term=ansi and +geometry))."
+    )
+
     return options_parser
 
 
