@@ -14,16 +14,18 @@ import re, pexpect, sys, time
 from byexample.common import constant
 from byexample.parser import ExampleParser
 from byexample.finder import ExampleFinder
-from byexample.runner import ExampleRunner, PexepctMixin, ShebangTemplate
+from byexample.runner import ExampleRunner, PexpectMixin, ShebangTemplate
 
 stability = 'experimental'
 
+
 class GDBPromptFinder(ExampleFinder):
     target = 'gdb-prompt'
 
     @constant
     def example_regex(self):
-        return re.compile(r'''
+        return re.compile(
+            r'''
             # Snippet consists of a single prompt line (gdb)
             (?P<snippet>
                 (?:^(?P<indent> [ ]*) \(gdb\)[ ]     .*)
@@ -35,19 +37,23 @@ class GDBPromptFinder(ExampleFinder):
                           (?![ ]*\(gdb\))  # Not a line starting with the prompt
                          .+$\n?            # But any other line
                       )*)
-            ''', re.MULTILINE | re.VERBOSE)
+            ''', re.MULTILINE | re.VERBOSE
+        )
 
     def get_language_of(self, *args, **kargs):
         return 'gdb'
 
     def get_snippet_and_expected(self, match, where):
-        snippet, expected = ExampleFinder.get_snippet_and_expected(self, match, where)
+        snippet, expected = ExampleFinder.get_snippet_and_expected(
+            self, match, where
+        )
 
         snippet = self._remove_prompts(snippet)
         return snippet, expected
 
     def _remove_prompts(self, snippet):
-        return snippet[6:]     # remove the (gdb) prompt
+        return snippet[6:]  # remove the (gdb) prompt
+
 
 class GDBParser(ExampleParser):
     language = 'gdb'
@@ -56,12 +62,12 @@ class GDBParser(ExampleParser):
     def example_options_string_regex(self):
         # anything of the form:
         #   #  byexample:  +FOO -BAR +ZAZ=42
-        return re.compile(r'#\s*byexample:\s*([^\n\'"]*)$',
-                                                    re.MULTILINE)
+        return re.compile(r'#\s*byexample:\s*([^\n\'"]*)$', re.MULTILINE)
 
     def process_snippet_and_expected(self, snippet, expected):
-        snippet, expected = ExampleParser.process_snippet_and_expected(self,
-                                            snippet, expected)
+        snippet, expected = ExampleParser.process_snippet_and_expected(
+            self, snippet, expected
+        )
         # remove any option string, gdb does not support
         # comments. If we do not do this, gdb will complain
         snippet = self.example_options_string_regex().sub('', snippet)
@@ -69,7 +75,7 @@ class GDBParser(ExampleParser):
         return snippet, expected
 
 
-class GDBInterpreter(ExampleRunner, PexepctMixin):
+class GDBInterpreter(ExampleRunner, PexpectMixin):
     language = 'gdb'
 
     def __init__(self, verbosity, encoding, **unused):
@@ -78,23 +84,25 @@ class GDBInterpreter(ExampleRunner, PexepctMixin):
         # --nh     do not read ~/.gdbinit
         # --nx     do not read any .gdbinit
         # --quiet  do not print version number on startup
-        PexepctMixin.__init__(self,
-                                PS1_re = r'\(gdb\)[ ]',
-                                any_PS_re = r'\(gdb\)[ ]')
+        PexpectMixin.__init__(
+            self, PS1_re=r'\(gdb\)[ ]', any_PS_re=r'\(gdb\)[ ]'
+        )
 
     def get_default_cmd(self, *args, **kargs):
-        return  "%e %p %a", {
-                    'e': "/usr/bin/env",
-                    'p': "gdb",
-                    'a': [
-                            "--nh",  # do not read ~/.gdbinit.
-                            "--nx",  # do not read any .gdbinit files in any directory
-                            "--quiet", # do not print version on startup
-                        ]
-                    }
+        return "%e %p %a", {
+            'e':
+            "/usr/bin/env",
+            'p':
+            "gdb",
+            'a': [
+                "--nh",  # do not read ~/.gdbinit.
+                "--nx",  # do not read any .gdbinit files in any directory
+                "--quiet",  # do not print version on startup
+            ]
+        }
 
     def run(self, example, options):
-        return PexepctMixin._run(self, example, options)
+        return PexpectMixin._run(self, example, options)
 
     def _run_impl(self, example, options):
         if not example.source:
@@ -105,10 +113,10 @@ class GDBInterpreter(ExampleRunner, PexepctMixin):
         if example.source.endswith('\n'):
             source = example.source[:-1]
 
-        return self._exec_and_wait(source, options)
+        return self._exec_and_wait(source, options, from_example=example)
 
     def interact(self, example, options):
-        PexepctMixin.interact(self)
+        PexpectMixin.interact(self)
 
     def initialize(self, options):
         shebang, tokens = self.get_default_cmd()
@@ -120,10 +128,14 @@ class GDBInterpreter(ExampleRunner, PexepctMixin):
         dfl_timeout = options['x']['dfl_timeout']
 
         # gdb will not print the address of a variable by default
-        self._exec_and_wait('set print address off\n', options, timeout=dfl_timeout)
+        self._exec_and_wait(
+            'set print address off\n', options, timeout=dfl_timeout
+        )
 
         # gdb will stop at the first null when printing an array
-        self._exec_and_wait('set print null-stop on\n', options, timeout=dfl_timeout)
+        self._exec_and_wait(
+            'set print null-stop on\n', options, timeout=dfl_timeout
+        )
 
         # gdb will not ask for "yes or no" confirmation
         self._exec_and_wait('set confirm off\n', options, timeout=dfl_timeout)
@@ -133,4 +145,3 @@ class GDBInterpreter(ExampleRunner, PexepctMixin):
 
     def cancel(self, example, options):
         return self._abort(example, options)
-
